from pandas import read_csv
import matplotlib.pyplot as plt
import numpy as np

# Names of the files:
calibration_table = "CalibCoeffs_logg.csv"# Name of the table we will use to calibrate
gaia_data = "Good_Ca_SolarN.csv" # Input data


# Read the coefficients of the calibration
calibdata = read_csv(calibration_table)

# Read the input gaia data and saves the useful parameters
gaiadata = read_csv(gaia_data)
logg = gaiadata["logg_gspspec"].values
mh_unc = gaiadata["mh_gspspec"].values # Uncalibrated [M/H]
alphafe_unc = gaiadata["alphafe_gspspec"].values # Uncalibrated [alpha/Fe]
cafe_unc = gaiadata["cafe_gspspec"].values # Uncalibrated [Ca/Fe]


#-------------------------------------------------
#This function is useful to apply the corrections:
def AbundanceCal(logg, coeffs, limits):
	'''Applies the polynomial correction defined by the coefficients stored in "coeffs" within the limits: limits[0]<=logg<limits[1].
	Note this function can be used with the calibrations based on the effective temperature (Teff) without any modification. '''
	# Impose maximum and minimum in logg
	logg = limits[0]*(logg<limits[0]) + limits[1]*(logg>limits[1]) + logg*(logg>=limits[0])*(logg<=limits[1])

	# Apply the correction
	correction = np.poly1d(np.flip(coeffs))(logg)# Note the flip function
	return(correction)
#-------------------------------------------------
 


# ------ METALLICITY [M/H] -----------------------
# Before correcting the individual abundances, let's correct the metallicity:
index_elm = list(calibdata["ID"]).index("MH") # Get the row for [M/H]
coeffs = calibdata[["p%i"%i for i in range(5)]].values[index_elm,:] # Read the coefficients.
limits = calibdata[["Minlogg", "Maxlogg"]].values[index_elm,:] # Read the validity limits.
mh_cal = mh_unc + AbundanceCal(logg, coeffs, limits); # Calcium with the 3rd degree calibration


# ------ CALCIUM [Ca/Fe] -----------------------
# Compute the 3rd-degree correction for the Calcium:
index_elm = list(calibdata["ID"]).index("CaFe") # Get the row for Calcium
coeffs = calibdata[["p%i"%i for i in range(5)]].values[index_elm,:] # Read the coefficients.
limits = calibdata[["Minlogg", "Maxlogg"]].values[index_elm,:] # Read the validity limits.
cafe_cal3 = cafe_unc + AbundanceCal(logg, coeffs, limits); # Calcium with the 3rd degree calibration

# Compute the 4th-degree correction for the Calcium:
index_elm = list(calibdata["ID"]).index("CaFe_2") # Get the row for Calcium (note _2 means 4th deg. correction)
coeffs = calibdata[["p%i"%i for i in range(5)]].values[index_elm,:] # Read the coefficients.
limits = calibdata[["Minlogg", "Maxlogg"]].values[index_elm,:] # Read the validity limits.
cafe_cal4 = cafe_unc + AbundanceCal(logg, coeffs, limits); # Calcium with the 4rd degree calibration



# ------ [Alpha/Fe] -----------------------
# Compute the 3rd-degree correction for the alpha:
index_elm = list(calibdata["ID"]).index("alphaFe") # Get the row for Alpha
coeffs = calibdata[["p%i"%i for i in range(5)]].values[index_elm,:] # Read the coefficients.
limits = calibdata[["Minlogg", "Maxlogg"]].values[index_elm,:] # Read the validity limits.
alphafe_cal3 = alphafe_unc + AbundanceCal(logg, coeffs, limits); # [alpha/Fe] with the 3rd degree calibration

# Compute the 4th-degree correction for the alpha:
index_elm = list(calibdata["ID"]).index("alphaFe_2") # Get the row for Alpha (note _2 means 4th deg. correction)
coeffs = calibdata[["p%i"%i for i in range(5)]].values[index_elm,:] # Read the coefficients.
limits = calibdata[["Minlogg", "Maxlogg"]].values[index_elm,:] # Read the validity limits.
alphafe_cal4 = alphafe_unc + AbundanceCal(logg, coeffs, limits); # [alpha/Fe] with the 4rd degree calibration




# ----------------------------------------------
#  PLOT THE RESULTS
# ----------------------------------------------
fig = plt.figure("Calibrations", figsize=(16.,9.))

# Density plot for the uncalibrated alpha
plt.subplot(321)
plt.hexbin( mh_unc, alphafe_unc, cmap="magma", bins="log", mincnt=1, extent=[-2,0.5,-0.4, 0.6])
plt.colorbar(label="Counts")
plt.xlabel("[M/H]$_{unc}$", fontsize=14)
plt.ylabel(r"[$\alpha$/Fe]$_{unc}$", fontsize=14)
plt.axis([-2,0.5,-0.4, 0.6])
# ------------


# Density plot for the uncalibrated calcium
plt.subplot(322)
plt.hexbin( mh_unc, cafe_unc, cmap="magma", bins="log", mincnt=1, label="Uncalib. Ca", extent=[-2,0.5,-0.4, 0.6])
plt.colorbar(label="Counts")
plt.xlabel("[M/H]$_{unc}$", fontsize=14)
plt.ylabel("[Ca/Fe]$_{unc}$", fontsize=14)
# ------------


# Density plot for the calibrated alpha (3-deg)
plt.subplot(323)
plt.hexbin( mh_cal, alphafe_cal3, cmap="magma", bins="log", mincnt=1, label="3rd deg. cal.", extent=[-2,0.5,-0.4, 0.6])
plt.colorbar(label="Counts")
plt.xlabel("[M/H]$_{cal3}$", fontsize=14)
plt.ylabel(r"[$\alpha$/Fe]$_{cal3}$", fontsize=14)
# ------------


# Density plot for the calibrated calcium (3-deg)
plt.subplot(324)
plt.hexbin( mh_cal, cafe_cal3, cmap="magma", bins="log", mincnt=1, label="3rd deg. cal.", extent=[-2,0.5,-0.4, 0.6])
plt.colorbar(label="Counts")
plt.xlabel("[M/H]$_{cal3}$", fontsize=14)
plt.ylabel("[Ca/Fe]$_{cal3}$", fontsize=14)
# ------------


# Density plot for the calibrated alpha (4-deg)
plt.subplot(325)
plt.hexbin( mh_cal, alphafe_cal4, cmap="magma", bins="log", mincnt=1, label="4rd deg. cal.", extent=[-2,0.5,-0.4, 0.6])
plt.colorbar(label="Counts")
plt.xlabel("[M/H]$_{cal4}$", fontsize=14)
plt.ylabel(r"[$\alpha$/Fe]$_{cal4}$", fontsize=14)
# ------------


# Density plot for the calibrated calcium (4-deg)
plt.subplot(326)
plt.hexbin( mh_cal, cafe_cal4, cmap="magma", bins="log", mincnt=1, label="4rd deg. cal.", extent=[-2,0.5,-0.4, 0.6])
plt.colorbar(label="Counts")
plt.xlabel("[M/H]$_{cal4}$", fontsize=14)
plt.ylabel("[Ca/Fe]$_{cal4}$", fontsize=14)
# ------------

plt.tight_layout()
plt.show()

